`timescale 1ns / 1ps

//******************************************************************************
//* Controller FSM for the stopwatch.                                          *
//* 1) Initially, the watch is stopped.                                        *
//* 2) When the button is pressed, the measurement starts.             <--|    *
//* 3) When the button pressed again, the measurement stops.              |    *
//* 4) When the button is pressed again, the measured time is cleared. ---|    *
//******************************************************************************
module controller_fsm(
   input  wire clk,              //16 MHz system clock
   input  wire rst,              //Reset signal
   input  wire btn,              //Button input
   output wire rst_timebase,     //Reset for the timebase
   output wire rst_counters      //Reset for the counters
);

//******************************************************************************
//* Detecting the rising edge on the button input.                             *
//******************************************************************************
//Synchronizing the button input with the system clock to avoid metastability.
(* shreg_extract = "no" *)
(* register_balancing = "no" *)
(* register_duplication = "no" *)
(* equivalent_register_removal = "no" *)
reg  [1:0] btn_sync_reg;
wire       btn_sync = btn_sync_reg[1];

always @(posedge clk)
begin
   btn_sync_reg <= {btn_sync_reg[0], btn};
end

//Generating the 100 Hz enable signal for sampling the button state.
reg  [17:0] clk_divider;
wire        clk_divider_tc = (clk_divider == 18'd0);

always @(posedge clk)
begin
   if (rst || clk_divider_tc)
      clk_divider <= 18'd159999;
   else
      clk_divider <= clk_divider - 18'd1;
end

//Rising edge detection.
reg  [1:0] btn_samples;
wire       btn_pressed = (btn_samples == 2'b01) & clk_divider_tc;

always @(posedge clk)
begin
   if (rst)
      btn_samples <= 2'b00;
   else
      if (clk_divider_tc)
         btn_samples <= {btn_samples[0], btn_sync};
end


//******************************************************************************
//* Controller FSM.                                                            *
//******************************************************************************
localparam CLEAR    = 2'd0;
localparam STOPPED1 = 2'd1;
localparam MEASURE  = 2'd2;
localparam STOPPED2 = 2'd3;

reg [1:0] state;

always @(posedge clk)
begin
   if (rst)
      state <= CLEAR;
   else
      case (state)
         //Complete the description of the FSM to implement the desired behavior.
         
         //Clearing the measured time by resetting the counters.
         CLEAR   : 
         
         //The measurement is stopped. Waiting for the button press.
         STOPPED1: 
         
         //Time is being measured. Stop when the button is pressed.         
         MEASURE : 
                      
         //The measurement is stopped. The display is
         //cleared when the button is pressed.           
         STOPPED2: 
      endcase
end

//Generating the reset signal for the timebase.
assign rst_timebase = 

//Generating the reset signal for the counters.
assign rst_counters = 


endmodule
